ad_page_contract {
    page to add a new file to the system

    @author Frank Bergmann (frank.bergmann@project-open.com)
    @creation-date 6 Nov 2018
    @cvs-id $Id$
} {
    project_id:integer
    path
    upload_file:trim,optional
    upload_file.tmpfile:tmpfile,optional
    creation_date
    modification_date
    sha1
    { upload_comment "" }
    { title "" }
    { description "description" }
}

set user_id [ad_conn user_id]
set n_bytes [file size ${upload_file.tmpfile}]
set package_id [ad_conn package_id]
set package_id [db_string package "select min(package_id) from apm_packages where package_key = 'file-storage'" -default $package_id]

ns_log Notice "upload: project_id=$project_id, path=$path, upload_file=$upload_file, tmp_file=${upload_file.tmpfile}"

set max_bytes [parameter::get_from_package_key -package_key "file-storage" -parameter "MaximumFileSize"]
if {$n_bytes > $max_bytes } {
    doc_return 507 "application/json" "{success: false, message: \"File is larger than the maximum size ($max_bytes bytes) allowed on this server\"}"
    ad_script_abort
}

ns_log Notice "upload: im_rest_fs_folder_for_path -object_id $project_id -path $path"
set folder_id [im_rest_fs_folder_for_path -object_id $project_id -path $path]
if {"" eq $folder_id} {
    # Folder path not found
    doc_return 400 "application/json" "{'success': false, 'message': 'Did not find folder ´[ns_quotehtml $path]´', 'data': {}}"
    ad_script_abort
}

# ToDo: Check permissions
# ToDo: Check for SHA1 in previous versions and deduplicate if necessary

set mime_type [cr_filename_to_mime_type -create -- $upload_file]
set file_id [db_nextval "acs_object_id_seq"]
ns_log Notice "upload: folder_id=$folder_id, file_id=$file_id, mime_type=$mime_type"


set file_version_id [fs::add_file \
        -name $upload_file \
        -item_id $file_id \
        -parent_id $folder_id \
        -tmp_filename ${upload_file.tmpfile} \
        -creation_user $user_id \
        -creation_ip [ad_conn peeraddr] \
        -title $upload_file \
        -description $description \
        -package_id $package_id \
        -mime_type $mime_type \
]

# ---------------------------------
# Fix the ]po[ object for cr_item
set file_id [db_string file_id "select item_id from cr_revisions where revision_id = :file_version_id" -default ""]
set file_exists_p [db_string p "select count(*) from im_files where file_id = :file_id"]
if {!$file_exists_p} {
    db_dml insert_file "
	insert into im_files (file_id, file_status_id, file_type_id) 
	values (:file_id, [im_file_status_active], [im_file_type_default])
    "
} else {
    # Nothing worth do update at the moment
}

# ---------------------------------
# Fix the ]po[ object for cr_revision
set file_version_exists_p [db_string p "select count(*) from im_file_versions where version_id = :file_version_id"]
if {!$file_version_exists_p} {
    db_dml insert_file_version "
	insert into im_file_versions (
		version_id, 
		version_status_id, 
		version_type_id, 
		version_upload_comment
	) values (
		:file_version_id, 
		[im_file_version_status_active],
		[im_file_version_type_default], 
		:upload_comment
	)
    "

    set update_sqls [list "version_sha1 = :sha1"]
    if {"" ne $creation_date && "undefined" ne $creation_date} { lappend update_sqls "version_creation_date = :creation_date" }
    if {"" ne $modification_date && "undefined" ne $modification_date} { lappend update_sqls "version_modification_date = :modification_date" }

    db_dml update_file_version "
	update im_file_versions set
		[join $update_sqls ",\n\t\t"]
	where
		version_id = :file_version_id
    "

} else {
    ad_return_complaint 1 "file_version update not implemented yet"
}

file delete ${upload_file.tmpfile}

doc_return 200 "application/json" "{success: true, message: \"Upload successful.\", data: {id: $file_id}}"
ad_script_abort

